/*	This PowerTap example is intentionally very simple so that the developer can
	focus on how to use the PowerTap Library.

	The basic steps are
		1) Initialize the PowerTap software with a call to InitPowerTap().
		2) Open a job by calling OpenJob()
		3) Submit tasks. Each task number is assigned by you and must be unique.
			Use SubmitTask() to do this.
		4) Call GetNextResult() to retrieve the results.
		5) Call PTIdle() frequently. This is where it all happens.

	A job is a long series of computations (or whatever) that can be broken down into
	many tasks.

	Other useful calls are
		1) CloseCurrJob() - Frees up slaves for helping others. Prepares for opening
			another new job.
		2) GetJobStatus() - Tells you how many tasks have been submitted, assigned
			and completed for the current job.
		3) GetTaskResult() - Similar to GetNextTask(), it gets a task result, but it
			gets a specified task result and returns false if that task is not ready.
			GetNextResult() is preferred because it delivers any result that is ready
			so that the calling program can begin processing the results as soon
			as possible.
		4) DisableSelf() - Frees up the machine running the main application. All of the
			tasks are assigned remotely. Completion is not assured while self is disabled.
		5) EnableSelf() - Ensures job completion by resuming assignments
			to the local machine.
		6)  and other optional functions.

	 1994 Always Thinking, Inc.    Glen Allen, Virginia  (800) 297-3888
*/

#include "PowerTap Lib.h"
#include "DoTask.h"

#define SPACE	32

// globals
Rect			nameRect,
				tRect;
Str32			ourMacName;

short	ColorFromName( Str32 name );
void	DrawResult( long taskNum, Handle resultHand, StringPtr macName );
void	DrawMacName( Str255 macName );

void main()
{
	long			i,				// general index.
					taskNo,			// task number of the task retrieved.
					jobLen,
					taskLen,
					resultLen,
					nDone;			// this is also provided by GetJobStatus().
	OSErr			myErr;
	Handle			theJobText,		// the job data for this example allocated by
					resultHand;		// the library, this will hold a result.
	TaskHandle		taskData;
	WindowPtr		ourWind;
	Str32			macName;		// name of the Macintosh that did the task.
	EventRecord		theEvt;
	StringHandle	localNameHand;	// the name of our mac in the Sharing Setup.
	unsigned long	theJobNum;		// randomly chosen job number.

	// MoreMasters should be called according to how many tasks and slaves are
	// expected to be involved in the largest job.
	// See the manual for details on figuring this.


	MaxApplZone();
	
	MoreMasters();
	MoreMasters();

	InitGraf( &qd.thePort );
	InitCursor();
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( nil );

	myErr = InitPowerTap( kGuestOrSecure );

	if ( ( myErr != noErr			) &&
		 ( myErr != kNoProgLinkErr	) &&		// no volunteers - no biggie.
		 ( myErr != kNoAppleTalkErr	) &&		// no remotes at all - were on our own.
		 ( myErr != kPreSystem7Err	) )			// no remotes at all - were on our own.
	{
		// a memory/resource shortage here is fatal. InitPowerTap() didn't need much.
		ClosePowerTap();
		ExitToShell();
	}
	
	// We are going to do a simple job where we have a block of text (job data)
	// and the slaves return one character from it for each task. We piece it together
	// and display it as it comes in.
	
	theJobText = GetResource( 'TEXT', 129 );
	if ( theJobText == nil )
	{
		ClosePowerTap();
		ExitToShell();
	}

	// get the name of our Macintosh
	ourMacName[0] = 0;
	localNameHand = GetString( -16413 );
	if ( localNameHand != nil )
	{
		HLock( (Handle) localNameHand );
		for ( i = 0; i <= (*localNameHand)[0]; i++ )
			ourMacName[i] = (*localNameHand)[i];
		HUnlock( (Handle) localNameHand );
		ReleaseResource( (Handle) localNameHand );
	}

	ourWind = GetNewWindow( 128, nil, (WindowPtr) -1 );
	if ( ourWind == nil )
	{
		ReleaseResource( theJobText );
		ClosePowerTap();
		ExitToShell();
	}
	else
		SetPort( ourWind );

	SetRect( &tRect, 5, 20, 505, 80 );
	FrameRect( &tRect );
	TextFont( courier );
	TextSize( 20 );

	jobLen = GetHandleSize( theJobText );

	// choose a job number that will be unique
	// _unless you want to re-use the previous job data_.
	theJobNum = TickCount();

	myErr = OpenJob( theJobNum, theJobText, jobLen );

	if ( myErr != noErr )
	{
		ClosePowerTap();
		ExitToShell();
	}

	// submit all of the tasks. they queue ~ LIFO.
	for ( i = jobLen-1; i >= 0L; i-- )
	{
		taskData = (TaskHandle) NewHandle( sizeof( TaskStruct ) );
		if ( taskData != nil )
		{
			(**taskData).taskType = findChar;
			(**taskData).offset = i;

			taskLen = sizeof( TaskStruct );
			myErr = SubmitTask( i, (Handle) taskData, taskLen, nil );
			if ( myErr != noErr )
				break;
		}
	}

	if ( myErr != noErr )
		jobLen = 0;			// get out of here

	// act on the task results as they come in
	nDone = 0L;
	while ( nDone < jobLen )
	{
		// get all of the results that are ready now.
		while ( GetNextResult( &taskNo, &resultHand, &resultLen, macName ) )
		{
			DrawResult( taskNo, resultHand, macName );
			nDone++;
		}

		// call PTIdle to give PowerTap some time to juggle the tasks.
		if ( PTIdle( 20L ) != noErr )
			break;

		WaitNextEvent( everyEvent, &theEvt, 15L, nil );
	}

	while ( !Button() )
		;
	while ( Button() )
		;

	// Because you retain access to the job & task data handles and the
	// task result handles, you are responsible for disposing of them.
	// NEVER dispose of a task data handle BEFORE closing the job.

	// This example does not keep track of the task data handles, so they are
	// left around. Leaving them around would waste memory and is NOT recommended.

	for ( taskNo = 0L; taskNo < jobLen; taskNo++ )
	{
		if ( GetTaskResult( taskNo, &resultHand, &resultLen, macName ) )
			DisposHandle( resultHand );
	}

	ClosePowerTap();

	ReleaseResource( theJobText );
	ReleaseResource( (Handle) ourWind );

	FlushEvents( everyEvent, 0 );

}	// main


void	DrawResult( long taskNum, Handle resultHand, StringPtr macName )
// draws the result. slaves are done in color and their name is noted.
{
	short	h, v;

	// we have an answer back. display the char.
	h = 10 + taskNum * CharWidth( 32 );
	v = tRect.bottom - 40;
	if ( taskNum > 40 )
	{
		h -= 41 * CharWidth( 32 );
		v += 20;
	}
	MoveTo( h, v );

	if ( EqualString( macName, ourMacName, TRUE, TRUE ) )
	{
		TextFace( normal );
		ForeColor( blackColor );
		FrameRect( &tRect );
	}
	else
	{
		if ( **resultHand == SPACE )
			TextFace( underline );		// make sure we know who did the space
		else
			TextFace( normal );
		ForeColor( ColorFromName( macName ) );
	}

	DrawChar( **resultHand );

	if ( !EqualString( macName, ourMacName, TRUE, TRUE ) )
	{
		DrawMacName( macName );
	}

}	// DrawResult

void	DrawMacName( Str255 macName )
{
	SetRect( &nameRect, 10, tRect.bottom - 15, 400, tRect.bottom - 5 );
	EraseRect( &nameRect );

	TextFont( monaco );
	TextSize( 9 );
	TextFace( normal );
	MoveTo( nameRect.left, nameRect.bottom-2 );
	DrawString( macName );
	TextFont( courier );
	TextSize( 20 );

}	// DrawMacName

short	ColorFromName( Str32 name )
{
	short	colors[6] = { yellowColor, greenColor, blueColor,
						  cyanColor, magentaColor, redColor };
	short	c,
			sum = 0;


	for ( c = 1; c <= name[0]; c++ )
		sum += name[c];

	sum = sum % 6;
	
	return colors[ sum ];

} // ColorFromName


