/*
		The PowerTap system provides an easy way for developers to add distributed processing
	to an application.  To distribute the tasks, System 7s Program Linking is required. It will
	still work correctly under System 6, but the work is all done locally.  The typical
	sequence for getting a job done is:

		1) Initialize the library
		2) Open the job, passing a handle to the data that is common to all tasks of the job.
		3) Submit the tasks, passing handles for task-specific data.
		4) Call PTIdle frequently to get the actual work done a little at a time.
		5) Request task results.
		6) Close up.

		In addition to the library providing the services above, the PowerTap system has two
	other components. The remote work is done by a faceless background application (FBA). It is
	provided as a project and the developer/user adds their DoTask() code to the project to
	build a Tap application specific to their needs. The FBA is controlled by a control panel,
	which launches it and controls any settings the user changes. Both of these must be customized
	by the developer - user. This includes customizing the PICTs, icons, strings, etc. The most
	important thing to change is the network identity. This is all documented in the manual and
	easy to do.

	Copyright 1994 Always Thinking, Inc.
	Written by Bruce Lawton.
	
	Sales and Tech Support:
	(800) 297-3888
	AOL: PowerTap
	Internet: powertap@aol.com
*/

#ifndef	__POWERTAP__
#define __POWERTAP__

// error codes
#define	kMaxTasksErr			1
#define	kDupTaskErr				2
#define	kNoCurrJobErr			3
#define	kBadNetNameErr			4
#define	kJobCancelledErr		5	// returned by PTIdle() when Command - . pressed
#define	kTimeOutErr				6
#define	kNoProgLinkErr			8
#define	kMissingResErr			9
#define	kNotFoundErr			10
#define	kCantCommErr			11
#define	kNoNameStrErr			12	// returned when the ppc name string is not found
#define	kPreSystem7Err			13
#define	kNoAppleTalkErr			14
#define	kRequiresJobClosedErr	15
#define	kInvalidTapSpecErr		16
#define	kNoThreadMgrErr			17


// security selectors for InitPowerTap() calls
#define	kOnlyGuest			1	// no username/password dialog is presented
#define	kGuestOrSecure		2	// dialog presented, Guest is allowed
#define	kOnlySecure			3	// dialog presented, Guest is disabled

// JobThreadParmBlock structs are only needed when using PowerTap via the Thread Mgr.
typedef	struct	JobThreadParmBlock {
	long			jobNo;
	unsigned long	sleepTicks;
	Boolean			yieldBetweenTasks;
} JobThreadParmBlock;

pascal	OSErr	InitPowerTap( short securitySelector );
//	Initializes the PowerTap library and verifies that AppleTalk is working.
//	Do not call this more than once because it could then leave memory fragmented.
//	After ClosePowerTap() is called, InitPowerTap() may be called normally with no
//	memory fragmentation.

pascal	OSErr	OpenJob( long jobNo, Handle jobDataHand, long jobDataLen );
//	Sets up a new job.

pascal	OSErr	SubmitTask( long		taskNo,
							Handle		taskDataHand,
							long		taskDataLen,
							StringPtr	targetTapName );
//	Returns noErr if the task was accepted. If the task is rejected, the reason code is
//	returned. If lacking memory, try again later.

pascal	OSErr	PTIdle( long sleepTicks );
//	This should be called frequently to ensure slaves are kept busy working
//	and lost tasks are reassigned promptly.
//	Allows the library to handle any traffic and make assignemnts.

pascal long	DoAllTasksThread( long threadParms );
//	Your main thread can create a NewThread, passing this DoAllTasksThread() as the
//	entry point. This thread will run until all tasks are done and will then terminate.
//	Any fatal errors are returned as a(long) OSErr. Your main thread yields to this
//	thread instead of calling PTIdle. Your DoTask() must still call PTIdle 6 times / sec.

pascal	Boolean	GetNextResult( long *taskNo, Handle *resultHand, long *resultLen, StringPtr macName );
//	Gets the result for the next task whose result is available.
//	Returns FALSE if none are available.
//	macName is the Chooser name of the Macintosh whose slave did the task.

pascal	Boolean	GetTaskResult( long taskNo, Handle *resultHand, long *resultLen, StringPtr macName );
//	Gets the result for the specified task. Returns FALSE if it is not available.
//	GetTaskResult is the preferred way to get results because you can handle the results as they
//	arrive rather than delaying, waiting for a specific task.

pascal	Boolean	GetTaskStats( long taskNo, unsigned long *submitTime,
										   unsigned long *assignTime,
										   unsigned long *computeTime,
										   unsigned long *completeTime, StringPtr macName );
//	Gets the timing stats for the specified task. Returns TRUE if the task is complete.
//	Times for submission, assignment and completion are based on TickCount for the local
//	Macintosh. Compute time is the interval, in ticks, that the task required.

pascal	void	CloseCurrJob( void );
//	Frees up straggling slaves for your next job or someone else's next job.

pascal	OSErr	GetJobStats( long jobNo, long *nTasksSubmitted, long *nTasksAssigned, long *nTasksDone );
//	Reports the number of tasks submitted, assigned and completed for the current job.

pascal	void	ClosePowerTap( void );
//	Closes network ports, disposes of all memory allocated by PowerTap.

pascal	void	DisableSelf( void );
//	Disables the self slave, unloading the local machine from doing work.
//	PTIdle() must still be called frequently to handle task traffic to & from the network.

pascal	void	EnableSelf( void );
//	Enables the self slave and ensures the job will complete.

pascal	OSErr	SetResultBufferSize( Size resultBufferSize );
//	The size for new task result allocations can be changed when there are no open jobs.

pascal	OSErr	SetMaxTaps( long maxTaps );
//	Allows programmer to choose _not_ to use the network or limit how much use.

///////////////////////////////////////////////////////////////////////////////////////////////
pascal	Boolean	DoTask( Handle taskData, long taskDataLen, Handle taskResult, long *resultLen );
//	Provided by you, the developer/user.
//	DoTask() is used in the main and slave (local and remote) applications.

pascal	OSErr	SetupJobData( void );
//	Provided by you, the developer/user.
//	SetupJobData() is used in the main and slave (local and remote) applications.
//	It is called once at the beginning of the job.

pascal	OSErr	CleanUpJobData( void );
//	Provided by you, the developer/user.
//	CleanUpJobData() is used in the main and slave (local and remote) applications.
//	It is called once at the end of the job.

pascal	Boolean	CommandDotPressed( void );
//	Returns TRUE when the user requests cancelling by pressing Command-. This does not mean
//	the calling app is in the foreground!

#endif
